#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include <libgen.h>

#define  STRLEN 200
void get_the_options(int ,char **),usage(char **),locerr(char *);
char prog[STRLEN];
int NMAX,tmax;

double rho(double *,int, int);
int main(int argc, char **argv){
  double *r,*tau,*x,norm;
  int     i,ndat=0,t,tcut;
  /*Default values for max number of data and maximum time for rho and tau*/
  /*Use -n and -t options to change them. */
  NMAX=2000000;tmax=1000;//NMAX=2000000 requires ~ 2e6*8=16MB of memory
  strcpy(prog,(char *)basename(argv[0]));//program name stored in prog
  get_the_options(argc,argv);
  /*Allocate the array where we store the input data:*/
  x = (double *)malloc( NMAX * sizeof(double)); //data
  /*Read data without exceeding NMAX as long as it is provided from stdin*/
  while( (ndat < NMAX) && (scanf("%lf",&x[ndat++]) == 1));
  if( ndat >= NMAX)
    fprintf(stderr,"# %s: Warning: Read ndat= %d and reached the limit %d\n",
	    prog,ndat,NMAX);
  /*we decrease tmax if it is comparable or larger to number of data.*/
  if(tmax > (ndat/10)) tmax = ndat/10;
  /*r[t] stores the values of autocorrelation function rho(t)*/
  r   = (double *)malloc( tmax * sizeof(double)); //autocorrelation function
  for(t=0;t<tmax;t++) r[t]  = rho(x,ndat,t);      //rho(t)
  norm=1.0/r[0];for(t=0;t<tmax;t++)r[t] *= norm;  //normalize r[0]=1
  /*tau[t] stores integrated autocorrelation times with tcut=t*/
  tau = (double *)malloc( tmax * sizeof(double)); //tau_int
  for(tcut=0;tcut<tmax;tcut++){
    tau[tcut] = 0.0;
    for(t=0;t<=tcut;t++) tau[tcut] += r[t];//sum of r[t]
  }
  /*  Output: an informational header and the results.*/
  printf("# ==========================================================\n");
  printf("#      Autoc function rho and integrated autoc time tau     \n");
  printf("# ndat= %d  tmax= %d\n",ndat,tmax);
  printf("# t         rho(t)              tau(tcut=t)                 \n");
  printf("# ==========================================================\n");
  /*Print the results in three columns:*/
  for(t=0;t<tmax;t++)
    printf("%d   %.15lg    %.15lg\n",t,r[t],tau[t]);
}/*main*/

double rho(double *x,int ndat,int t){
  int n,t0;
  double xav0=0.0,xavt=0.0,r=0.0;
  n=ndat-t;
  if(n<1)locerr("rho: n<1");
  /*Calculate the two averages: xav0= <x>_0 and xavt= <x>_t */
  for(t0=0;t0<n;t0++){
    xav0 += x[t0];
    xavt += x[t0+t];
  }
  xav0/=n;xavt/=n; //normalize the averages to number of data
  /*Calculate the t-correlations:*/
  for(t0=0;t0<n;t0++)
    r += (x[t0]-xav0)*(x[t0+t]-xavt);
  r/=n;            //normalize the averages to number of data
  return r;
}
void locerr( char *errmes ){
  fprintf(stderr,"%s: %s Exiting....\n",prog,errmes);
  exit(1);
}
#define OPTARGS  "?h1234567890.t:n:"
void get_the_options(int argc,char **argv){

  int c,errflg = 0;
  while (!errflg && (c = getopt(argc, argv, OPTARGS)) != -1){
    switch(c){
    case 'n':
      NMAX = atoi(optarg);
      break;
    case 't':
      tmax = atoi(optarg);
      break;
    case 'h':
      errflg++;/*call usage*/
      break;
    default:
      errflg++;
    }/*switch*/
    if(errflg) usage(argv);
  }/*while...*/
}/*get_the_options()*/
void usage(char **argv){
  fprintf(stderr,"\
Usage: %s  [-t <maxtime>] [-n <ndata>]\n\
      Reads data from stdin (one column) and computes autocorrelation\n\
      function and integrated autocorrelation time.\n",prog);
  exit(1);
}/*usage()*/
