/*****************            boot.c       *********************************
 Simple program for computation of errors of data using bootstrap.
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include <libgen.h>
#include <time.h>

#define  MAXDAT 1000000
#define  STRLEN 200
char prog[STRLEN];
int SAMPLES,maxdat;

void bootstrap(int,int,double *,double *,double *,double *,double *);
void get_the_options(int ,char **),usage(char **),locerr(char *);
double drandom();
void   init_random();
int main(int argc,char **argv){
  int    ndat=0;
  double O,dO,chi,dchi;
  double *x;
  strcpy(prog,(char *)basename(argv[0]));
  maxdat=-1;SAMPLES=1000;
  get_the_options(argc,argv);init_random();
  if( maxdat <= 0 ) maxdat = MAXDAT;
  x   = (double *) calloc((size_t) maxdat,(size_t) sizeof(double));
  /*Read data from stdin as long as it fits the array x[].
    ndat counts the number of data.*/
  while( scanf("%lg",&x[ndat])==1 ){
    ndat++;
    if(ndat == maxdat){//we give user a warning if we ignore some data
      fprintf(stderr,"# %s: Warning: Read ndat= %d. Reached the limit %d\n",
	      prog,ndat,maxdat);
      break;           //break the while loop
    }
  }
  bootstrap(ndat,SAMPLES,x,&O,&dO,&chi,&dchi);//calculate....
  printf("# NDAT = %d data. SAMPLES = %d groups\n",ndat,SAMPLES);
  printf("# <o>, chi= (<o^2>-<o>^2)\n");
  printf("# <o> +/- err                             chi +/- err\n");
  printf("%.15lg %.15lg %.15lg %.15lg\n",O,dO,chi,dchi);
}
/****************************************************************/
/* bootstrap routine: you can use it in any of your programs....*/
/****************************************************************/
void bootstrap(int ndat, int samples, double *x, 
	  double *avO, double *erO, double *avchi, double *erchi){

  int i,j,k,binw,bin;
  double *O,*O2,*chi,xs;
  O    = (double *) calloc((size_t) samples  ,(size_t) sizeof(double));
  O2   = (double *) calloc((size_t) samples  ,(size_t) sizeof(double));
  chi  = (double *) calloc((size_t) samples  ,(size_t) sizeof(double));
  for(j=0;j<samples;j++){
    for(i=0;i<ndat;i++){
      k = (int)(ndat*drandom());
      O [j] += x[k];
      O2[j] += x[k]*x[k];
    }
    O[j]  /=ndat;O2[j]  /=ndat;
    chi[j] = O2[j] - O[j]*O[j];
  }
  /* Compute averages: */
  *avO=0.0;*avchi=0.0;
  for(j=0;j<samples;j++){
    *avO += O[j]; *avchi += chi[j];
  }
  *avO /= samples;*avchi /= samples;
  /* Compute errors:   */
  *erO = 0.0; *erchi = 0.0;
  for(j=0;j<samples;j++){
    *erO   += (O  [j]-*avO  )*(O  [j]-*avO  );
    *erchi += (chi[j]-*avchi)*(chi[j]-*avchi);
  }
  *erO /= samples   ; *erchi /= samples;
  *erO  = sqrt(*erO); *erchi  = sqrt(*erchi);
  /* Compute the real avO */
  *avO = 0.0;
  for(i=0;i<ndat;i++) *avO += x[i];
  *avO /= ndat;
  free(O);free(chi);//release memory back to the system
}
/****************************************************************/
void locerr( char *errmes ){
  fprintf(stderr,"%s: %s Exiting....\n",prog,errmes);
  exit(1);
}

#define OPTARGS  "?hs:d:"
void get_the_options(int argc,char **argv){

  int c,errflg = 0;
  while (!errflg && (c = getopt(argc, argv, OPTARGS)) != -1){
    switch(c){
    case 's':
      SAMPLES = atoi(optarg);
      break;
    case 'd':
      maxdat = atoi(optarg);
      break;
    case 'h':
      errflg++;/*call usage*/
      break;
    default:
      errflg++;
    }/*switch*/
    if(errflg) usage(argv);
  }/*while...*/
}/*get_the_options()*/

void usage(char **argv){
  fprintf(stderr,"\
Usage: %s  [options] <file>\n\
       -s  : No. samples Def. 1000\n\
       -d  : Give the maximum number of data points read.Def. %d\n\
Computes <o>, chi= (<o^2>-<o>^2)\n\
Data is in one column from stdin.\n",prog,MAXDAT);
  exit(1);
}/*usage()*/
/************************  drandom() *****************************/
#define a 16807
#define m 2147483647
#define q 127773
#define r 2836
#define conv (1.0/(m-1))
#include <sys/types.h>
#include <unistd.h>
long seed;
double drandom(){
  long l;
  l    = seed/q;
  seed = a*(seed-q*l) - r*l;
  if(seed < 0) seed +=m;
  return conv*(seed-1);
}
#include <unistd.h>
#include <sys/types.h>
/* Scramble time using the pid number:*/
void init_random(){
  long pid;
  seed = (long) time( (time_t) 0);
  pid  = (long) getpid(); //Process ID number
  seed = seed ^ (pid + (pid << 15));
  printf("# Initiated drandom() with seed= %d\n",seed);
}
