/************************ options.c ***********************************/
#include "include.h"
char prog[1024];

/*Get the options function: See "man 3 getopt" for usage*/
/*Option letters are defined with this string*/
#define OPTARGS  "?h1234567890.L:b:s:S:n:w"
void get_the_options(int argc,char **argv){
  int c,errflg = 0;

  strcpy(prog,(char *)basename(argv[0]));
  while (!errflg && (c = getopt(argc, argv, OPTARGS)) != -1){
    switch(c){
    case 'w':
      algorithm = 1;
      break;
    case 'L':
      L = atoi(optarg);
      break;
    case 'b':
      beta  = atof(optarg);
      break;
    case 's':
      start = atoi(optarg);
      break;
    case 'S':
      seed  = atol(optarg);
      break;
    case 'n':
      nsweep = atoi(optarg);
      break;
    case 'h':
      errflg++;/*call usage*/
      break;
    case '?':
      errflg++;/*call usage*/
      break;
    case '1':case '2':case '3':case '4':case '5':case '6':case '7':case '8':
    case '9':case '0':case '.':
      /*optind--;*//*it was not meant to be an option*/
      return;
    default:
      errflg++;
    }/*switch*/
    if(errflg) usage(argv);
  }/*while...*/


}/*get_the_options()*/

void usage(char **argv){

  /*Careful: New lines end with \n\ : No space after last backslacsh
    indicates l ine is broken....*/
  fprintf(stderr,"\
Usage: %s  [options]                                                \n\
       -L: Lattice length (N=L*L)                                   \n\
       -b: beta  (options beta overrides the one in config)         \n\
       -s: start (0 cold, 1 hot, 2 old config.)                     \n\
       -S: seed  (options seed overrides the one in config)         \n\
       -n: number of sweeps and measurements of E and M             \n\
       -w: use wolff algorithm for the updates                      \n\
Monte Carlo simulation of 2d Ising Model. Metropolis is used by     \n\
default, cluster algorithms can be chosen with options. Using the   \n\
options, the parameters of the simulations must be set for a new run\n\
(start=0,1). If start=2, a configuration is read from the file conf.\n",prog);
// printed usage message "
  exit(1);

}/*usage()*/ 

void locerr( char *errmes ){ 

  fprintf(stderr,"%s: %s Exiting....\n",prog,errmes); 
  exit(1); 

}


/*print a message about the simulation: */
#include <time.h>
void simmessage(FILE *fp){

  char USER[100],HOST[100],MACH[100];
  time_t t;

  time(&t);/* store time in seconds in *t. see: "man 2 time"  */
  strcpy(USER,getenv("USER"));
  strcpy(MACH,getenv("HOSTTYPE"));
  strcpy(HOST,getenv("HOST"));

  fprintf(fp,"\
# ###################################################################\n\
#      2d Ising Model with Metropolis algorithm on square lattice    \n\
# Run on %s ( %s ) by %s on %s#                                      \n\
# L       = %d (Lattice linear dimension, N=L*L)                     \n\
# seed    = %ld (random number gener. seed)                          \n\
# nsweeps = %d (No. of sweeps)                                       \n\
# beta    = %f                                                       \n\
# start   = %d (0 cold, 1 hot, 2 old config)\n",
          HOST,MACH,USER,ctime(&t),
          L,seed,nsweep,beta,start);
  fflush(fp);
}/* message()*/
